/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/
#include <xc.h>         /* XC8 General Include File */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
//#include <stdlib.h>
#include <stdio.h>

#include "i2c.h"
#include "system.h"

void I2C_Init (void)
{
//    PORTCbits.RC4 = 1;      // SDA
//    PORTCbits.RC3 = 1;      // SCL
    // Initialise les registres du PIC18F26K22 en mode I2C Master
    // I2C Master Clock Speed: 64000000 / ((4 * (SSPADD + 1)) = 64000000/(4*(159+1))
    SSPCON1 = 0x28;      // Enable SDA and SCL, I2C Master mode
    SSPCON2 = 0x00;      // Reset MSSP Control Register
    SSPADD = 159;         // Standard I2C Clock speed: 159 -> 100 kHz & 39 -> 400kHz
    SSPSTAT = 0xC0;      // Slew Rate is disable for 100 kHz mode
    PIR1bits.SSPIF=0;    // Clear MSSP Interrupt Flag
}

void I2C_Idle(void)
{
    // Wait I2C Bus and Status Idle (i.e. ACKEN, RCEN, PEN, RSEN, SEN)
    while (( SSPCON2 & 0x1F ) || ( SSPSTATbits.R_nW));
}

void I2C_Start(unsigned char stype)
{
    I2C_Idle();                     // Ensure the I2C module is idle
    if (stype == START)
    {
        SSPCON2bits.SEN = 1;          // Start I2C Transmission
        while(SSPCON2bits.SEN);
    }
    else
    {
        SSPCON2bits.RSEN = 1;         // ReStart I2C Transmission
        while(SSPCON2bits.RSEN);
    }
}

void I2C_Stop(void)
{
    I2C_Idle();                   // Ensure the I2C module is idle
    SSPCON2bits.PEN = 1;                // Stop I2C Transmission
    while(SSPCON2bits.PEN);
}

void I2C_Write(unsigned char data)
{
    I2C_Idle();                   // Ensure the I2C module is idle
  // Send the Data to I2C Bus
    SSPBUF = data;
    if (SSPCON1bits.WCOL)         // Check for write collision
        return;

    while(SSPSTATbits.BF);        // Wait until write cycle is complete
}

char I2C_Read (char flag)
{
    uint8_t buffer=0;
    //Ensure the I2C module is idle
    I2C_Idle();
    //Enable Receive Mode
    SSPCON2bits.RCEN = 1;   //Enable master for 1 byte reception
    while (!SSPSTATbits.BF);    //Wait until buffer is full
    buffer = SSPBUF;
    
    if (flag == 0)      //Send acknowledge to continue reading
    {
        SSPCON2bits.ACKDT = 0; //Ack bit, 0 = Ack byte received
        SSPCON2bits.ACKEN = 1; //Start Ack sequence
        //while (SSP2CON2bits.ACKEN);
    }
    else        //No Ack to stop reading
    {
        SSPCON2bits.ACKEN = 1;  //Start Ack sequence
        SSPCON2bits.ACKDT = 1;  //Ack bit, 1 = No Ack byte received
        //while (SSP2CON2bits.ACKEN);
    }
    return(buffer);
}
//  Scanner le bus
//  Cette fonction crit l'adresse d'un esclave sur le bus.
//  Si un esclave est pr"sent  l'adresse, il rpond par un ACK.

uint8_t i2c_adresse;
uint8_t i2c_data;

void Scan_IC_Bus (void)
{
    for (i2c_adresse = 0x10; i2c_adresse < 0xF0; i2c_adresse+=2)
    {
        I2C_Start(START);
        I2C_Write(i2c_adresse);
        __delay_us(200);
        if(SSPCON2bits.ACKSTAT == 0)
        {
            printf("I2C Adr : 0x%X\r\n",i2c_adresse);
        }
        I2C_Stop();
        __delay_ms(1);
    }
}