/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/
#include <xc.h>         /* XC8 General Include File */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "system.h"
#include "line.h"

uint8_t i, k;

void Init_Ligne(void)
{
    for (k=0; k<8; k++)
    {
        capteur[k].min = 255;
        capteur[k].max = 0;
    }
}

void Lecture_Ligne(void)
{
    Sel_Canal_ADC(0);
    __delay_us(50);
    capteur[0].val = Read_ADC_8b();         // Probleme de lecture
    Sel_Canal_ADC(1);
    __delay_us(50);
    capteur[1].val = Read_ADC_8b();
    Sel_Canal_ADC(2);
    __delay_us(20); 
    capteur[2].val = Read_ADC_8b();
    Sel_Canal_ADC(3);
    __delay_us(20);
    capteur[3].val = Read_ADC_8b();
    Sel_Canal_ADC(4);
    __delay_us(20);
    capteur[4].val = Read_ADC_8b();
    Sel_Canal_ADC(0);
}

void Line_Calibrate(void)
{
    for (k=0; k<5; k++)
    {
        if (capteur[k].val < capteur[k].min)
            capteur[k].min = capteur[k].val;
        if (capteur[k].val > capteur[k].max)
            capteur[k].max = capteur[k].val;
        capteur[k].delta = (uint8_t)(capteur[k].max - capteur[k].min);
        capteur[k].corr = (uint8_t)(((capteur[k].val - capteur[k].min)*100)/capteur[k].delta);
    }
}

uint8_t Get_Capt_Corr(uint8_t select)
{
    return capteur[select].corr;
}

int16_t num, denom;
int16_t pos;

int16_t Get_Pos(void)
{
//    for (k=0; k<5; k++)
//    {
//        if (capteur[k].val > capteur[k].min)        // pas de valeur ngative
//            capteur[k].corr = (uint8_t)(((capteur[k].val - capteur[k].min)*100)/capteur[k].delta);
//        else
//            capteur[k].corr = 0;
//    }
    denom = 0;
//    denom = capteur[0].corr + capteur[1].corr + capteur[2].corr + capteur[3].corr;
//    denom = denom + capteur[4].corr + capteur[5].corr + capteur[6].corr + capteur[7].corr;
    for (k=0; k<5; k++)
    {
        denom = denom + capteur[k].corr;
    }
//    num = 0;
//    num = 10*capteur[0].corr + 20*capteur[1].corr + 30*capteur[2].corr + 40*capteur[3].corr;
//    num = num + 50*capteur[4].corr + 60*capteur[5].corr + 70*capteur[6].corr + 80*capteur[7].corr;
////    for (k=0; k<8; k++)
////    {
////        num = num + (capteur[k].corr * (k+1)*10);
////    }
    pos = (int16_t) num/denom;
    pos = pos - 45;
    return pos;
}

uint16_t Get_Somme(void)
{
    return (uint16_t)denom;
}
//===============================================
void Print_Ligne(uint8_t entite)
{
    if(entite == 0)
    {
        for (k=0; k<5; k++)
        {
            printf("%d\t",capteur[k].val);
        }
    }
    else if(entite == 1)
    {
        for (k=0; k<5; k++)
        {
            printf("%d\t",capteur[k].corr);
        }
    }
    else if(entite == 2)
        printf("Pos %d",pos );
    PutString("\r\n");
}
//
void Line_Eeprom_Write(void)
{
    i = 0;
    for (k=0; k<5; k++)
    {
        Eeprom18_Write (i++,capteur[k].min);
        Eeprom18_Write (i++,capteur[k].max);
        Eeprom18_Write (i++,capteur[k].delta);
    }
}
////
void Line_Eeprom_Read(void)
{
    i = 0;
    for (k=0; k<5; k++)
    {
        capteur[k].min = Eeprom18_Read(i++);
        capteur[k].max = Eeprom18_Read(i++);
        capteur[k].delta = Eeprom18_Read(i++);
    }
}

unsigned char Eeprom18_Read(unsigned int adresse)
{
    while (EECON1bits.WR);
    EEADR = adresse & 0xFF;
//    EEADRH = (adresse >> 8) & 0x03;
    EEPGD = 0;
    EECON1bits.RD = 1;
    while (EECON1bits.RD);
    return EEDATA;
}

void Eeprom18_Write(unsigned int adresse, unsigned char valeur)
{
char Save_IT_State;    
    while (EECON1bits.WR);
    EEADR = adresse & 0xFF;
//    EEADRH = (adresse >> 8) & 0x03;
    EEDATA = valeur;
    EECON1bits.EEPGD = 0;
    EECON1bits.CFGS = 0;
    WREN = 1;
    Save_IT_State = INTCONbits.GIE;
    INTCONbits.GIE = 0;
    EECON2 = 0x55;
    EECON2 = 0xAA;
    EECON1bits.WR = 1;
    while ((EECON1bits.WR == 1) && (EECON1bits.WRERR == 0));
    WREN = 0;
    while(valeur != Eeprom18_Read(adresse));
    INTCONbits.GIE = Save_IT_State;
}

//  Dtection de la marque finale
//  Petite machine d'tats pour filtrer
//  les croisements et la marque de dpart
uint8_t etat_stop = 0;

uint8_t Fonct_Stop(void)
{
uint8_t stop_value = 0;

    switch(etat_stop)
    {
//  Dtection de la marque de dpart       
        case 0:
            if(STOP == 1)
                etat_stop = 1;
        break;
        case 1:
            if(STOP == 0)
                etat_stop = 2;
        break;
//  Croisement ou marque finale        
        case 2:
            if(denom > 300)
            {
                etat_stop = 0;
            }
            if(STOP == 1)
            {
                etat_stop = 3;
            }
            break;
//  Marque finale
        case 3:
            if(STOP == 0)
            {
                stop_value = 1;
                etat_stop = 0;
            }
            break;
            
        default:
            etat_stop = 0;
            break;
    }
    return stop_value;
}