
#include <xc.h>
#include <stdint.h>                     // uint8_t def
#include <stdio.h>
#include <stdlib.h>

#include "system.h"

/*******************************************************************************
*   Initialisations du C
*******************************************************************************/
void Init_System(void)
{
    OSCCON = 0b01110000;       // Oscillateur interne en mode 64MHz
    OSCTUNEbits.PLLEN = 1;              // PLL active
    
    ANSELA = 0;                         // Toutes les lignes analogiques bloques
    ANSELB = 0;
    ANSELC = 0;
    TRISA = 0xFF;                       // Toutes les lignes en entres
    TRISB = 0xFF;
    TRISC = 0xFF;
    
    DEL_VERTE = 0;
    DIR_DEL_VERTE = 0;
    DEL_ROUGE = 0;
    DIR_DEL_ROUGE = 0;
    
    
    WPUBbits.WPUB4 = 1;
    INTCON2bits.nRBPU = 0;
    DIR_BP = 1;
    
    DIR_SENS_A1 = 0;
    DIR_SENS_A2 = 0;
    DIR_SENS_B1 = 0;
    DIR_SENS_B2 = 0;

    SENS_A1 = 0;
    SENS_A2 = 0;
    SENS_B1 = 0;
    SENS_B2 = 0;
    
//    DIR_PIEZZO = 0;
//    
//    DIR_I2C_SCL = 1;
//    DIR_I2C_SDA = 1;
    
//  Initialisation Timer0
    T0CONbits.T08BIT = 0;           // En Mode 16-bit compteur
    T0CONbits.T0CS = 0;
    T0CONbits.PSA = 0;              // Prescaler vers Timer0
    T0CONbits.T0PS = TMR0_PRE;      // Choix du Prescaler
//    TMR0H = TMR0_VALUE >> 8;        // Prchargement du Timer0
//    TMR0L = TMR0_VALUE & 0xFF;      // en mode 2 fois 8bits
    TMR0 = TMR0_VALUE;              // Prchargement en mode 8bits
    INTCONbits.TMR0IF = 0;          // Effacement du "flag" du Timer0
    T0CONbits.TMR0ON = 1;           // Timer0 Actif
    
//  Initialisation USART1      Sync = 0, BRGH = 1, BRG16 = 1
    SPBRG = 138;			// 138 pour 115200 baud @ 64MHz
//    SPBRG = 69;			// 69 pour 230400 baud @ 64MHz (pas stable))
    BAUDCONbits.BRG16 = 1;
    RCSTA1 = 0x90;
    TXSTA1 = 0x24;

//  Initialisation USART2      Sync = 0, BRGH = 0, BRG16 = 0
    SPBRGH2 = 6;
    SPBRG2 = 136;            // 1666 pour 9600 baud @ 64MHz
    BAUDCON2bits.BRG16 = 1;
    RCSTA2 = 0x90;
    TXSTA2 = 0x24;          // TXEN = 1, Sync = 0, BRGH = 0

//  Initialisation Timer1
//   Mode Encodeur
//    T1CONbits.TMR1CS = 0b10;        // Source externe
//    T1CONbits.T1CKPS = 0;           // 1:1 Prescaler
//    TMR1 = 0;
//    T1CONbits.TMR1ON = 1;           // Timer1 Actif

//   Mode Mesure UltraSon
//    T1CONbits.TMR1CS = 0b00;           // Source interne 64MHz /4 = 16MHz
//    T1CONbits.T1CKPS = 0b11;           // 1:8 Prescaler => Increment 62.5 x 8 = 500ns
//    T1CONbits.T1RD16 = 0;
////    TMR1 = 0;                       // 
//    T1CONbits.TMR1ON = 0;           // Timer1 bloqu
    
//  Initialisation Timer3               Mesure dure de la boucle while(1)
//    T3CONbits.TMR3CS = 0b00;        // Source interne 64MHz /4 = 16MHz
//    T3CONbits.T3CKPS = 0b11;           // 1:8 Prescaler => Increment 62.5 x 8 = 500ns
////    TMR3 = 0;
//    T3CONbits.TMR3ON = 0;           // Timer3 bloqu
    
//  Initialisation Timer2  
//  Pour ECCP Freq: 5kHz -> Periode: 200s
    T2CONbits.T2CKPS = 0b10;        // 1:16 Prescaler
    PR2 = 199;                      // 200s = 62.5 x 16 x 200
    T2CONbits.TMR2ON = 1;           // Timer2 Actif
    
//  Initialisations ECCP
    CCP1CON = 0b00001100;           // Forward, Actif bas
    CCP2CON = 0b00001100;           // CCP2 sur P2A
    CCP3CON = 0b00001100;           // CCP3 sur P2A
    CCP4CON = 0b00001100;
    CCPTMRS0bits.C1TSEL = 0;        // Slection Timer2 pour CCP1
    CCPTMRS0bits.C2TSEL = 0;        // Slection Timer2 pour CCP2
    CCPTMRS0bits.C3TSEL = 0;        // Slection Timer2 pour CCP3
    CCPTMRS1bits.C4TSEL = 0;        // Slection Timer2 pour CCP4
    CCPR1L = 0;
    CCPR2L = 0;                   // rapport cyclique  0%
    CCPR3L = 0;
    CCPR4L = 0;
    
//  Initialisation Timer5 
//  Pour gnrer des notes de frq: 220Hz -> 5kHz
//    T5CONbits.TMR5CS = 0b00;        // 62.5ns
//    T5CONbits.T5CKPS = 0b11;        // 1:8 Prescaler
//    T5CONbits.TMR5ON = 0;           // Timer5 Bloqu
    
//  Initialisation CCP5
//  Gnration son par Compare mode sur CCP5
//    CCP5CON = 0b0010;      // Toggle la sortie
//    CCPTMRS1bits.C5TSEL = 0b10;     // Slection Timer5 pour CCP5
//    CCPR5L = 0;
    
//  Initialisation ADC
    ANSELA = 0b00010111;    // AN0, AN1, AN2, AN3, AN4
    ADCON0bits.CHS=0;       // AN0
    ADCON1bits.PVCFG=0;     // Rf+ -> AVdd
    ADCON1bits.NVCFG=0;     // Rf- -> AVss
    ADCON2bits.ADFM=0;      // Left justified -> 8 bits
//    ADCON2bits.ADFM=1;      // Right justified -> 10 bits
    ADCON2bits.ACQT = 0b101;      // 12 Tad
    ADCON2bits.ADCS = 0b110;      // Fosc/64
    ADCON0bits.ADON=1;      // ADC enable
    
//  Initialisations des Interruptions
    IPEN = 1;
    TMR0IE = 0;
    RCIE = 1;           // UART1 Receive Interrupt Enable
    RC1IP = 0;          // UART1 Receive Low Priority
    TX1IE = 1;          // UART1 Transmit Interrupt Enable
    TX1IP = 0;          // // UART1 Receive Interrupt EnableUART1 Transmit Low Priority
    RC2IE = 1;          // UART2 Receive Interrupt Enable
    RC2IP = 0;          // UART2 Receive Low Priority
    TMR2IF = 0;
    TMR4IF = 0;
    TMR4IP = 1;
    TMR4IE = 1;
//    RBIF = 0;
//    RBIE = 1;           // Interruption sur RB0 -> INT0
    INT0IF = 0;
    INT0IE = 0;         // Interruption sur RB0 -> INT0
    INTEDG0 = 1;        // Front montant
    PEIE = 1;
    ei();
}

void putch(unsigned char caract)
{
    while(!TX1IF);
    TX1REG = caract;
}

void PutString(const char *chaine)
{
    while((*chaine) != 0)       // Tant qu'il y a un caractre  envoyer
    {
        putch(*chaine);
        if (*chaine == 10) 
            putch(10);
        if (*chaine == 13) 
            putch(13);
        chaine++;               // Incrmente le pointer
    }
}

//uint16_t Encodeur_T1(void)
//{
//    return TMR1;
//}
//
//uint16_t Encodeur_T3(void)
//{
//    return TMR3;
//}

void Sel_Canal_ADC(uint8_t canal)
{
    ADCON0bits.CHS = canal;
}

uint8_t Read_ADC_8b(void)
{
    ADCON0bits.GO_nDONE = 1;
    while (ADCON0bits.GO_nDONE);
    return ADRESH;
}

//uint16_t Read_ADC_10b(void)
//{
//    ADCON0bits.GO_nDONE = 1;
//    while (ADCON0bits.GO_nDONE);
//    return ADRESH<<8 | ADRESL;
//}
